***********************************************
****Chapter 2: Variance-components models *****
***********************************************

*** 2.2 How reliable are peak-expiratory-flow measurements?
use http://www.stata-press.com/data/mlmus2/pefr, clear

* plot data
generate mean_wm = (wm1+wm2)/2
summarize mean_wm

twoway (scatter wm1 id, msymbol(circle))                                ///
       (scatter wm2 id, msymbol(circle_hollow)),                        ///
       xtitle(Subject id) xlabel(1/17) ytitle(Mini Wright measurements) ///
       legend( order(1 "Occasion 1" 2 "Occasion 2")) yline(453.9118)    
       

*** 2.5 Estimation using Stata

list if id<6, clean noobs

* stack repeated observations for same method into single variable
reshape long wp wm, i(id) j(occasion)
list if id<6, clean noobs

* using xtreg
xtreg wm, i(id) mle

* using xtmixed
xtmixed wm || id:, mle

* using gllamm
gllamm wm, i(id) nip(12) adapt
estimates store RI


*** 2.6 Hypothesis tests and confidence intervals

* for between-cluster variance
quietly xtmixed wm || id:, mle
estimates store m1
quietly xtmixed wm, mle
lrtest m1 .

* using score test
ssc install xtvc
quietly xtreg wm, mle
xtvc

* F-test
xtreg wm, i(id) fe


*** 2.7 More on statistical inference

* model-based and robust SE's for OLS

regress wm
regress wm, vce(cluster id)


*** 2.8 Crossed versus nested effects

* random intercept model with occasion 2 dummy

generate occ2 = occasion==2
xtmixed wm occ2 || id:, mle

*xtreg wm occ2, i(id) mle
*gllamm wm occ2, i(id) nip(12) adapt


*** 2.9 Assigning values to the random intercepts

** 2.9.1 Maximum likelihood estimation
* via OLS
quietly xtmixed wm || id:, mle
predict pred, xb
generate res = wm - pred

quietly tabulate id, generate(subject)
regress res subject1-subject17, noconstant

* via mean total residual
egen ml = mean(res), by(id)
display ml[1]

*** 2.9.2 Empirical Bayes prediction
* using shrinkage factor
display 107.05^2/(107.05^2+(19.91^2)/2)
generate eb1 = .98299832*ml

* using xtmixed
quietly xtmixed wm || id:, mle
predict eb2, reffects
sort id
format eb1 eb2 %8.2f
list id eb1 eb2 if occasion==1, clean noobs

*** 2.9.3 Empirical Bayes variances
estimates restore RI
gllapred eb, u

* posterior and prediction error standard errors
display ebs1[1]

* marginal standard errors
display sqrt(.98299832*107.05^2)

* conditional standard errors
display sqrt(.98299832*(1-.98299832)*107.05^2)


