************************************************************
****Chapter 3: Random-intercept models with covariates *****
************************************************************

*** 3.2 Does smoking during pregnancy affect birthweight?

use http://www.stata-press.com/data/mlmus2/smoking, clear

* data structure
xtsum birwt smoke black, i(momid)


*** 3.4 Estimation using Stata

xtreg birwt smoke male mage hsgrad somecoll collgrad       ///
  married black kessner2 kessner3 novisit pretri2 pretri3, ///
  i(momid) mle

xtmixed birwt smoke male mage hsgrad somecoll collgrad     ///
  married black kessner2 kessner3 novisit pretri2 pretri3  ///
  || momid:, mle

gllamm birwt smoke male mage hsgrad somecoll collgrad      ///
  married black kessner2 kessner3 novisit pretri2 pretri3, ///
  i(momid) adapt
estimates store gllamm


*** 3.5 Coefficients of determination or variance explained

xtreg birwt, i(momid) mle

xtreg birwt hsgrad somecoll collgrad married black, i(momid) mle


*** 3.6 Hypothesis tests

* Wald test
quietly xtreg birwt smoke male mage hsgrad somecoll collgrad  ///
  married black kessner2 kessner3 novisit pretri2 pretri3, ///
  i(momid) mle
testparm kessner2 kessner3

* Likelihood ratio test
estimates store full
quietly xtreg birwt smoke male mage hsgrad somecoll collgrad   ///
  married black novisit pretri2 pretri3, i(momid) mle
lrtest full .

* Predicted means and CIs
generate education = hsgrad*1 + somecoll*2 + collgrad*3
label define ed 0 "no HS grad" 1 "HS grad" 2 "some Coll" 3 "Coll grad"
label values education ed
estimates restore full
adjust male=0 mage married=1 black=0 kessner2=0 kessner3=0 ///
   novisit=0 pretri2=0 pretri3=0, by(smoke education) ci format(%4.0f)

*** 3.7 Between and within effects

xtreg birwt smoke male mage hsgrad somecoll collgrad       ///
  married black kessner2 kessner3 novisit pretri2 pretri3, ///
  i(momid) be

xtreg birwt smoke male mage hsgrad somecoll collgrad       ///
  married black kessner2 kessner3 novisit pretri2 pretri3, ///
  i(momid) fe

egen mn_smok = mean(smoke), by(momid)
generate dev_smok = smoke - mn_smok

xtreg birwt dev_smok mn_smok male mage hsgrad somecoll     ///
  collgrad married black kessner2 kessner3 novisit pretri2 ///
  pretri3, i(momid) mle

lincom mn_smok - dev_smok


* Means for other time-varying covariates 
egen mn_male = mean(male), by(momid)
egen mn_mage = mean(mage), by(momid)
egen mn_kessner2 = mean(kessner2), by(momid)
egen mn_kessner3 = mean(kessner3), by(momid)
egen mn_novisit = mean(novisit), by(momid)
egen mn_pretri2 = mean(pretri2), by(momid)
egen mn_pretri3 = mean(pretri3), by(momid)

/*
foreach var of varlist male mage kessner* novisit pretri* {
   egen mn_`var' = mean(`var'), by(momid)
}
*/

xtreg birwt smok mn_smok male mn_male mage mn_mage hsgrad somecoll   ///
  collgrad married black kessner2 mn_kessner2 kessner3 mn_kessner3   ///
  novisit mn_novisit pretri2 mn_pretri2 pretri3 mn_pretri3, i(momid) mle

testparm mn_*

xtreg birwt smok mn_smok male mage mn_age hsgrad somecoll  ///
  collgrad married black kessner2 kessner3 novisit pretri2 ///
  pretri3, i(momid) mle

* Hausman endogeneity test

quietly xtreg birwt smoke male mage hsgrad somecoll collgrad  ///
  married black kessner2 kessner3 novisit pretri2 pretri3,    ///
  i(momid) fe
estimates store fixed

quietly xtreg birwt smoke male mage hsgrad somecoll collgrad  ///
  married black kessner2 kessner3 novisit pretri2 pretri3,    ///
  i(momid) re
estimates store random

hausman fixed random


*** 3.9 Residual diagnostics

estimates restore gllamm
gllapred lev2, ustd
gllapred lev1, pearson

histogram lev1, normal xtitle(Standardized level-1 residuals)

histogram lev2m1 if idx==1, normal xtitle(Standardized level-2 residuals)

gllamm, robust

