**********************************************
****Chapter 4: Random-coefficient models *****
**********************************************

*** 4.3 Separate linear regressions for each school

use http://www.stata-press.com/data/mlmus2/gcse, clear

regress gcse lrt if school==1
predict p_gcse, xb 
twoway (scatter gcse lrt) (line p_gcse lrt, sort) if school==1, ///
  xtitle(LRT) ytitle(GCSE)

egen num = count(gcse), by(school)
statsby inter=_b[_cons] slope=_b[lrt], by(school) saving(ols): ///
  regress gcse lrt if num>4 
sort school
merge school using ols
drop _merge

twoway scatter slope inter, xtitle(Intercept) ytitle(Slope)

egen pickone = tag(school)
summarize inter slope if pickone==1
correlate inter slope if pickone==1, covariance

generate pred = inter + slope*lrt

sort school lrt
twoway (line pred lrt, connect(ascending)), xtitle(LRT) ///
   ytitle(Fitted regression lines)


*** 4.5 Estimation using Stata

** 4.5.1 Using xtmixed 
xtmixed gcse lrt || school:, mle
estimates store ri

xtmixed gcse lrt || school: lrt,  cov(unstructured) mle
estat recovariance
estimates store rc

** 4.5.2 Using gllamm 

gllamm gcse lrt, i(school) adapt
estimates store rig

eq slope: lrt
generate cons = 1
eq inter: cons

matrix a = e(b)
matrix a = (a,0,0)
gllamm gcse lrt, i(school) nrf(2) eqs(inter slope) ip(m) nip(15) ///
  adapt from(a) copy
estimates store rcg


*** 4.6 Testing the slope variance

lrtest ri rc


*** 4.7 Interpretation of estimates

twoway function sqrt(9.0447+2*0.1804*x+0.0145*x^2+55.3653), range(-30 30) ///
 xtitle(LRT) ytitle(Estimated standard deviation of total residual)


*** 4.8 Assigning values to the random intercepts and slopes

** 4.8.1 Maximum likelihood estimation

estimates restore rc
predict fixed, xb
generate totres = gcse - fixed
statsby mli=_b[_cons] mls=_b[lrt], by(school) saving(ols, replace): ///
  regress totres lrt
sort school
merge school using ols
drop _merge

list lrt gcse mli mls if school==48, clean noobs

** 4.8.2 Empirical Bayes prediction

estimates restore rc
predict ebs ebi, reffects

list school mli ebi mls ebs if pickone==1 & (school<10 | school==48), noobs

twoway (scatter ebi mli if pickone==1 & school!=48, mlabel(school)) ///
  (function y=x, range(-10 10)), legend(off) xtitle(ML estimate)    ///
  ytitle(EB prediction) legend(off)

twoway (scatter ebs mls if pickone==1 & school!=48, mlabel(school)) ///
  (function y=x, range(-0.6 0.6)), xtitle(ML estimate)              ///
  ytitle(EB prediction) legend(off)

** 4.8.3 Model visualization
predict murc, fitted

sort school lrt
twoway (line murc lrt, connect(ascending)), xtitle(LRT) ///
  ytitle(Empirical Bayes regression lines for model 2)

estimates restore ri
predict muri, fitted
sort school lrt
twoway (line muri lrt, connect(ascending)), xtitle(LRT) ///
  ytitle(Empirical Bayes regression lines for model 1)

estimates restore rcg
gllapred murcg, linpred

** 4.8.4 Residual diagnostics

predict res1, residuals

histogram reff1 if pickone==1, normal xtitle(Predicted random slopes)

histogram reff2 if pickone==1, normal xtitle(Predicted random intercepts)

histogram res1, normal xtitle(Predicted level-1 residuals)

** 4.8.5 Inferences for individual schools

estimates restore rcg  
gllapred reff, u

gllapred murcg, linpred

gsort + reffm1 -f
generate rank = sum(f)
generate labpos = reffm1 + 1.96*reffs1 + .4
serrbar reffm1 reffs1 rank if f==1, addplot(scatter labpos rank,     ///
  mlabel(school) msymbol(none) mlabpos(0)) scale(1.96) xtitle(Rank)  ///
  ytitle(Prediction) legend(off)


*** 4.9 Two-stage formulation 

tabulate schgen, generate(w)

rename w2 boys 
rename w3 girls
generate boys_lrt = boys*lrt
generate girls_lrt = girls*lrt

xtmixed gcse lrt boys girls boys_lrt girls_lrt || school: lrt, ///
   cov(unstructured) mle

