*****************************************************************
****Chapter 5: Longitudinal, panel, and growth-curve models *****
*****************************************************************


*** 5.2 How and why do wages change over time?

use http://www.stata-press.com/data/mlmus2/wagepan, clear


*** 5.3 Data structure
** 5.3.1 Missing data

xtdescribe if lwage<., i(nr) t(year)

** 5.3.2 Time-varying and time-constant variables

xtsum lwage union educ year, i(nr)


*** 5.5 Random and fixed effects approaches

** 5.5.1 Correlated rsiduals

generate educt = educ - 12
generate yeart = year - 1980

quietly regress lwage black hisp union married exper yeart educt
predict res, residuals

* Variances and correlations (run 6 lines in one block)
preserve
keep nr res yeart
reshape wide res, i(nr) j(year)
tabstat res*, statistics(variance) format(%4.3f) 
correlate res*, wrap
restore

** 5.5.2 Fixed-intercept model
* Using xtreg
xtreg lwage black hisp union married exper yeart educt, i(nr) fe

* Using anova
generate ethnic = black*1 + hisp*2

anova lwage ethnic / nr|ethnic yeart, continuous(yeart)

xi: xtreg lwage i.ethnic, i(nr) be

xtreg lwage yeart, i(nr) fe

display ((10.6791909+653.797374)/544)/.125918924

** 5.5.3 Random intercept model

xtmixed lwage black hisp union married exper yeart educt || nr:, mle
estimates store ri

** 5.5.4 Random coefficient model

xtmixed lwage black hisp union married exper yeart educt || nr: exper, ///
   cov(unstructured) mle
estimates store rc

lrtest ri rc


** 5.6.2 Marginal modeling using Stata

xtgee lwage black hisp union married exper yeart educt, i(nr) ///
   t(yeart) correlation(ar 1) vce(robust)
estat wcorrelation, format(%4.3f)


*** 5.7 Autoregressive or lagged response models

by nr (yeart), sort: generate lag1 = lwage[_n-1]
sort nr yeart
list nr yeart lwage lag1 in 1/9, clean noobs

regress lwage lag1 black hisp union married exper yeart educt


*** 5.9 Missing data

clear
set obs 10000
set seed 1323232
generate zeta = invnormal(uniform())
generate y1 = 2 + zeta + invnormal(uniform())
generate y2 = 2 + zeta + invnormal(uniform())

replace y2 = . if y1>2 & uniform()<0.9

tabstat y1 y2

generate id = _n
reshape long y, i(id) j(occasion)
quietly tabulate occasion, generate(occ)

xtreg y occ1 occ2, noconstant i(id) mle


*** 5.10 How do children grow?

use http://www.stata-press.com/data/mlmus2/asian, clear

** 5.10.1 Observed growth trajectories

label define g 1 "Boy" 2 "Girl"
label values gender g
sort id age

graph twoway (line weight age, connect(ascending)), ///
 by(gender) xtitle(Age in years) ytitle(Weight in Kg)

** 5.11.1 Random-intercept model 

generate age2 = age^2
xtmixed weight age age2 || id:, mle

** 5.11.2 Random-coefficient model 

xtmixed weight age age2 || id: age, cov(unstructured) mle
generate girl = gender - 1
xtmixed weight age age2 girl || id: age, cov(unstructured) mle

** 5.11.3 Two-stage formulation

generate age_girl = age*girl
xtmixed weight age age2 girl age_girl|| id: age, cov(unstructured) mle


*** 5.12 Prediction of trajectories for individual children

quietly xtmixed weight age age2 girl || id: age, cov(unstructured) mle
predict traj, fitted

twoway (scatter weight age) (line traj age, sort) if girl==1, ///
 by(id, compact legend(off))

*** 5.13 Prediction of mean trajectory and 95% band

estat recovariance

twoway (function Weight = _b[_cons] + _b[age]*x + _b[age2]*x^2,        /// 
   range(0.1 2.6) lwidth(medium))                                      ///
       (function upper = _b[_cons] + _b[age]*x + _b[age2]*x^2          ///
    + 2*sqrt(0.354+2*0.0476*x+0.2598*x^2), range(0.1 2.6) lpat(dash))  ///
       (function lower = _b[_cons] + _b[age]*x + _b[age2]*x^2          ///
    - 2*sqrt(0.354+2*0.0476*x+0.2598*x^2), range(0.1 2.6) lpat(dash)), ///     
     legend(order(1 "Mean" 2 "Limits of 95% trajectory band"))         ///
     xtitle(Age in years) ytitle(Weight in Kg)


*** 5.14 Complex level-1 variation or heteroskedasticity

generate boy = 1 - girl
eq het: girl boy

generate cons = 1
eq inter: cons
eq slope: age

gllamm weight age age2 girl, i(id) nrf(2) eqs(inter slope) s(het) adapt

display exp(-.706*2)
display exp(-.447*2)

display chi2tail(1,2*(253.87-252.41))

